import json
from typing import Generator

from fastapi import Depends, HTTPException, status
from fastapi.security import OAuth2PasswordBearer
from jose import jwt
from pydantic import ValidationError
from sqlalchemy.orm import Session

import crud, models, schemas
from core import security
from core.config import settings
from db.session import SessionLocal
from models.authority import Authority

reusable_oauth2 = OAuth2PasswordBearer(
    tokenUrl=f"{settings.API_V1_STR}/user/login/"
)


def get_db() -> Generator:
    try:
        db = SessionLocal()
        yield db
    finally:
        db.close()


def get_current_user(token: str = Depends(reusable_oauth2)
                     ) -> schemas.UserDBBase:
    # def get_current_user(token: str
    #                      ) -> schemas.UserDBBase:
    try:
        payload = jwt.decode(
            token, settings.SECRET_KEY, algorithms=[security.ALGORITHM]
        )
        user = schemas.UserDBBase(**payload)
    except (jwt.JWTError, ValidationError):
        raise HTTPException(
            status_code=status.HTTP_403_FORBIDDEN,
            detail="Could not validate credentials",
        )
    if not user:
        raise HTTPException(status_code=404, detail="User not found")
    return user


def get_current_active_user(
        current_user: models.User = Depends(get_current_user),
) -> models.User:
    if not crud.user.is_active(current_user):
        raise HTTPException(status_code=400, detail="Inactive user")
    return current_user


def get_current_active_superuser(
        current_user: models.User = Depends(get_current_user),
) -> models.User:
    if not crud.user.is_superuser(current_user):
        raise HTTPException(
            status_code=400, detail="The user doesn't have enough privileges"
        )
    return current_user


def check_project(project_id: int, db: Session = Depends(get_db)):
    if not crud.project.get(db, id=project_id):
        raise HTTPException(status_code=404, detail="没有这个项目")
    return project_id


def check_tree(db, *, project_id: int,
               user_id: int,
               space_id: int = None,
               folder_id: int = None,
               dashboard_id: int = None,
               **kwargs
               ) -> schemas.Tree:



    where = [
        Authority.project_id == project_id,
        Authority.user_id == user_id
    ]

    if space_id is not None:
        where.append(Authority.space_id == space_id)

    if folder_id is not None:
        where.append(Authority.folder_id == folder_id)

    if dashboard_id is not None:
        where.append(Authority.dashboard_id == dashboard_id)

    if not db.query(Authority).filter(*where).first():
        raise HTTPException(status_code=406, detail='请检查参数')

    return schemas.Tree(project_id=project_id,
                        space_id=space_id,
                        folder_id=folder_id,
                        dashboard_id=dashboard_id
                        )
